local ui = {}
local vars = dofile'vars.lua'
local cc = dofile'ColorPicker.lua'

local ENABLE_COLORS={OFF='\127707070', ON='\127ffffff'}

--Begin UI --

ui.colorpicker = cc.create_colorpicker_dialog{
	title="Chose a flash alert color",
	default_name="Off",
	default_color="0 0 0",
}

-- 0-1 => 0-1, 2-100 => 0.02-1, default 1
local function get_volume(value)
	local vol = tonumber(value)
	
	if not vol then vol = 1
	elseif vol >=0 and vol <= 1 then
		--no conversion necessary
	elseif vol > 1 and vol <= 100 then
		vol = vol/100
	else vol = 1 end
	
	return vol
end

local sorted_alerts = {}
ui.element = {}
ui.element.flash_toggle = iup.stationtoggle{title="Enable flash alerts", value=vars.is_flash_alert and "ON" or "OFF"}
ui.element.sound_toggle = iup.stationtoggle{title="Enable sound alerts", value=vars.is_sound_alert and "ON" or "OFF"}
for id,v in ipairs(vars.alerts) do
	local enable = v.enable and "ON" or "OFF"
	local flash_color = v.flash_color or "0 0 0"
	
	ui.element[id] = {}
	ui.element[id].label = iup.label{title=ENABLE_COLORS[enable]..v.title.."\127o", } --active=enable, expand="HORIZONTAL"}
	ui.element[id].option = v.option_title~=nil and iup.stationtoggle{title=v.option_title, value=v.option_enable and "ON" or "OFF", active=enable} or iup.fill{}
	ui.element[id].data = v.data~=nil and v.data~='' and iup.text{value=v.data, size="%6x", active=enable} or iup.fill{size=2}
	ui.element[id].popup = iup.list{dropdown="YES", active=enable}
	--ui.element[id].wav = iup.stationtoggle{title="wav", value=v.wav, active=enable} --TODO: allow .wav
	ui.element[id].vol_label = iup.label{title="\127ffffffvolume\127o"}
	ui.element[id].volume = iup.text{value=v.vol or '100', size="%4x", active=enable}
	ui.element[id].button = iup.stationbutton{
		title="Test",
		active=enable,
		action=function()
			gksound.GKPlaySound(vars.sound_name[tonumber(ui.element[id].popup.value)], get_volume(ui.element[id].volume.value)) --TODO:fix sound name
		end
	}
	ui.element[id].checkbox = iup.stationtoggle{
		title="",
		value=enable,
		action=function(self)
			local active = self.value
			
			if iup.GetType(ui.element[id].option)~='fill' then ui.element[id].option.active = active end
			if iup.GetType(ui.element[id].data)~='fill' then ui.element[id].data.active = active end
			
			--TODO clean-up
			ui.element[id].popup.active = active
			--ui.element[id].wav.active = active
			ui.element[id].volume.active = active
			ui.element[id].button.active = active
			ui.element[id].colorcanvas.active = active
			
			ui.element[id].label.title = ENABLE_COLORS[active]..filter_colorcodes(ui.element[id].label.title)
			--ui.element[id].popup.title = ENABLE_COLORS[active]..filter_colorcodes(ui.element[id].popup.title)
			
			iup.Refresh(iup.GetDialog(self))
		end,
	}
	ui.element[id].color_label = iup.label{title=ENABLE_COLORS[flash_color=="0 0 0" and "OFF" or "ON"].."Flash\ncolor\127o"}
	ui.element[id].colorbox = iup.label{title="", bgcolor=flash_color, expand="YES"}
	ui.element[id].colorcanvas = iup.canvas{active=enable, bgcolor="0 0 0", button_cb=function()
		ui.colorpicker.color = ui.element[id].colorbox.bgcolor
		ui.colorpicker.color_cb = function(self, color)
			ui.element[id].colorbox.bgcolor=color
			ui.element[id].color_label.title=ENABLE_COLORS[color=="0 0 0" and "OFF" or "ON"].."Flash\ncolor\127o"
		end
		
		ShowDialog(ui.colorpicker, iup.CENTER, iup.CENTER)
	end}
	
	--populate list of sounds
	for n=1,vars.NUM_SOUNDS_MAX do
		iup.SetAttribute(ui.element[id].popup,n,'\127ffffffsound'..n..(v.wav and ".wav" or ".ogg"))
	end
	ui.element[id].popup.value = v.sound
	
	--create hbox with parameters for each alert
	ui.element[id].box = iup.hbox{
		ui.element[id].checkbox,
		ui.element[id].label,
		iup.fill{},
		ui.element[id].option,
		ui.element[id].data,
		ui.element[id].popup,
		--ui.element[id].wav, --TODO
		ui.element[id].vol_label,
		ui.element[id].volume,
		ui.element[id].button,
		ui.element[id].color_label,
		iup.stationhighopacitysubframe{
			iup.zbox{
				ui.element[id].colorbox,
				ui.element[id].colorcanvas,
				expand="YES", ALL="YES",
			},
			size="36x36",
			expand="NO",
		},
		iup.fill{size="%2x"}, --hack to get entries to fill list
		gap=8,
		--size="HALFx",
		expand="HORIZONTAL",
		rank=v.rank, --non-standard attribute
	}
	
	table.insert(sorted_alerts, ui.element[id].box)
end
table.sort(sorted_alerts, function(a,b) return tonumber(a.rank)<tonumber(b.rank) end)

function ui:ShowHelp()
	local help_str ='Sound Alert Suite v'..SoundAlertSuite.version..' Help\n\n'..
[[Use this window to change the settings for Sound Alert Suite that are saved to the config file. Each row in the alert list corresponds to an action or event in the game that triggers a sound to be played by the plug-in.

*The ']]..'\127ffff00enable flash alerts\127o'..[[' checkbox causes the screen to flash when the alert is triggered (only if a non-black color is selected for the alert). Good to use if your sound is turned down low.
From the console use: ]]..'\127a0a0a0/SAS flash on/off/toggle\127o\n'..[[
*Use the ']]..'\127ffff00enable sound alerts\127o'..[[' checkbox to easily toggle sound playback without having to manually set all sound volumes to zero and back again.
From the console use: ]]..'\127a0a0a0/SAS sound on/off/toggle\127o\n'..[[

Each entry in the list of alerts has the following components:
*The checkbox on the far left enables alerts for the given event when checked.
*Additional checkboxes and data fields may appear right of the event description, which can be used to set additional parameters relevant to that event.
*The sound to play is chosen from the pop-up box. You can replace the sound files with your own if you choose. Valid file names are "sound1.ogg" to "sound]]..vars.NUM_SOUNDS_MAX..[[.ogg", and the sound files should reside in the /plugins/SoundAlertSuite/sounds/ directory.
*You can set the playback volume to a number from 0 (silent) to 1 (loudest). Values between 1 and 100 are also valid and are treated as a percentage.
*The Test button plays the sound selected in the pop-up so you can hear what it sounds like at the specified volume.
*The flash color swatch opens a color picker when clicked. This lets you choose the color to flash the screen (only when HUD is visible) when the event occurs. Setting the color to black (0 0 0) disables the flash.

Supported events:
]]
--Create rest of help dynamically so that ranking order is correct (and so I don't forget to add new alerts to the list =P)
--also to hide events that are not applicable (e.g. required plug-in not installed)

	local sorted_alerts = {}
	for _,info in ipairs(vars.alerts) do
		local help = tostring(info.help)
		local title = tostring(info.title)
		
		if help and title and not info.hide then
			table.insert(sorted_alerts, {rank=info.rank, title=title, help=help})
		end
	end
	table.sort(sorted_alerts, function(a,b) return tonumber(a.rank)<tonumber(b.rank) end) --TODO consolidate into single function
	
	for id,info in ipairs(sorted_alerts) do
		help_str = help_str..id..')\t\127ffff00'..info.title..'\127o -- '..info.help..'\n'
	end
	StationHelpDialog:Open(help_str)
end

local button_help = iup.stationbutton{title="Help", hotkey=iup.K_h, action=ui.ShowHelp}
--iup.hbox{iup.stationbutton{title="Help", hotkey=iup.K_h, action=function() OpenHelp() end}, iup.fill{}, closeb, cancelb}
local button_ok = iup.stationbutton{title="Close"}
--local button_cancel = iup.stationbutton{title="Cancel"}
local button_defaults = iup.stationbutton{title="Defaults"}
button_defaults.action = function()
	local retval
	QuestionDialog:SetMessage(
[[Confirmation:
Are you sure you want to reset to the default settings for SoundAlertSuite?]],
		"Yes", function() retval=1 return iup.CLOSE end,
		"No", function() retval=2 return iup.CLOSE end
	)
	PopupDialog(QuestionDialog, iup.CENTER, iup.CENTER)
	
	if retval==1 then
		vars.ResetDefaults()
		ui.Update(button_defaults)
	end
end

local event_list = iup.stationsubsublist{
	--unpack(sorted_alerts),
	expand="YES",
	--size="%1x1",
	--size = "400x400",
	control = "YES",
}
local even, odd, alpha = ListColors[0], ListColors[1], ListColors.Alpha
local max_size
local sorted_sub = {}
for i,entry in ipairs(sorted_alerts) do
	local subdlg = iup.dialog{
		entry,
		size="%80x",
		--expand="HORIZONTAL",
		border="NO",
		menubox="NO",
		resize="NO",
	}
	if math.fmod(i, 2) == 0 then
		subdlg.bgcolor = even.." "..alpha
	else subdlg.bgcolor = odd.." "..alpha end
	
	--subdlg:map()
	--local size = subdlg.size
	--if (not max_size) or max_size < size then max_size = size end
	
	table.insert(sorted_sub, subdlg)
end


for _,subdlg in ipairs(sorted_sub) do
	--subdlg.size = max_size
	iup.Append(event_list, subdlg)
end
event_list:map()
	

local frame = iup.vbox{
	iup.hbox{iup.fill{}, iup.label{title="Sound Alert Suite",font=Font.H3}, iup.fill{}},
	iup.hbox{iup.fill{}, iup.label{title="Version "..SoundAlertSuite.version}}, --GOAL: remove global use
	iup.fill{size=10},
	iup.hbox{iup.fill{}, ui.element.sound_toggle, iup.fill{size="%14"}, ui.element.flash_toggle},
	iup.hbox{iup.label{title="Alert List"}, iup.fill{}},
	event_list,
	alignment="ALEFT",
	--iup.label{title=""},
	--iup.hbox{button_help, iup.fill{}, button_ok},
}
iup.Append(frame, iup.label{title=""})
iup.Append(frame, iup.hbox{button_help, button_defaults, iup.fill{}, button_ok, gap=8})

-- Create Dialog --

ui.dialog = iup.dialog{
	iup.vbox{
		iup.fill{},
		iup.hbox{
			iup.fill{},
			iup.stationhighopacityframe{
				iup.stationhighopacityframebg{
				--iup.pdarootframe{
				--iup.stationsubframe{
					frame,
					expand="NO",
					size="%80x%85",
					gap=8,
				},
			},
			iup.fill{},
		},
		iup.fill{},
	},
	BGCOLOR = "0 0 0 160 *",
	FULLSCREEN="YES",
	RESIZE="NO",
	BORDER="NO",
	MENUBOX="YES",
	TOPMOST="YES",
	DEFAULTESC=button_ok,
	DEFAULTENTER=button_ok
}
ui.dialog:map()

-- Display Dialog --

function ui:Show()
	if not ui.dialog then ui:Init() end
	
	ui.Update(self) --reset control values in case of changed by console

	ShowDialog(ui.dialog,iup.CENTER,iup.CENTER)
	iup.Refresh(ui.dialog)
end

--capture state of controls and save to config
function ui:Hide()
	for id,v in ipairs(vars.alerts) do
		local old_enable = v.enable
		v.enable = ui.element[id].checkbox.value=="ON" --get new value from dialog
		
		--Register/Unregister events if enabled/disabled --GOAL: Don't use global SoundAlertSuite
		if old_enable ~= v.enable then
			for _,event_name in ipairs(v.events) do
				if v.enable then
					if not SoundAlertSuite.events_index[event_name] then --event not already registered
						RegisterEvent(SoundAlertSuite, event_name)
						SoundAlertSuite.events_index[event_name] = {}
					end
					table.insert(SoundAlertSuite.events_index[event_name], id)
				else
					--remove all instances of this alert from the event's table
					if SoundAlertSuite.events_index[event_name] then --check table is not nil
						for i=#SoundAlertSuite.events_index[event_name],1,-1 do --have to traverse table backwards for table.remove()
							if SoundAlertSuite.events_index[event_name][i]==id then table.remove(SoundAlertSuite.events_index[event_name], i) end
						end
						
						--unregister event if no more alerts are using the event
						if #SoundAlertSuite.events_index[event_name]==0 then
							SoundAlertSuite.events_index[event_name] = nil
							UnregisterEvent(SoundAlertSuite, event_name)
						end
					end
				end
			end
			
			if v.init then --call init function because state changed
				SoundAlertSuite[v.init](v.enable)
			end
		end
		
		--save dialog settings
		v.sound = tonumber(ui.element[id].popup.value)
		v.data = v.data and ui.element[id].data.value
		v.option_enable = v.option_enable~=nil and ui.element[id].option.value=="ON"
		v.vol = get_volume(tonumber(ui.element[id].volume.value))
		v.flash_color = ui.element[id].colorbox.bgcolor
	end
	
	vars.is_flash_alert = ui.element.flash_toggle.value=="ON"
	vars.is_sound_alert = ui.element.sound_toggle.value=="ON"
	
	vars.WriteConfig()
	HideDialog(ui.dialog)
end

--Set all control values to config values
--TODO consolidate with above code
function ui:Update()
	local activate_ctrls = {'label', 'popup', 'volume', 'button', 'colorcanvas'} --controls to set active status
	
	for id,v in ipairs(vars.alerts) do
		local enable = v.enable and "ON" or "OFF"
		local flash_color = v.flash_color or "0 0 0"
		
		for _,key in ipairs(activate_ctrls) do ui.element[id][key].active = enable end
		
		ui.element[id].label.title = ENABLE_COLORS[enable]..v.title.."\127o"
		ui.element[id].volume.value = v.vol or '100'
		ui.element[id].checkbox.value = enable
		ui.element[id].color_label.title = ENABLE_COLORS[flash_color=="0 0 0" and "OFF" or "ON"].."Flash\ncolor\127o"
		ui.element[id].colorbox.bgcolor = flash_color
		ui.element[id].popup.value = v.sound
		
		if v.option_title~=nil then
			ui.element[id].option.value = v.option_enable and "ON" or "OFF"
			ui.element[id].option.active = enable
		end
		if v.data~=nil and v.data~='' then
			ui.element[id].data.value = v.data
			ui.element[id].data.active = enable
		end
	end
	
	ui.element.flash_toggle.value = vars.is_flash_alert and "ON" or "OFF"
	ui.element.sound_toggle.value = vars.is_sound_alert and "ON" or "OFF"
end

function button_ok.action() ui:Hide() end

return ui,vars